--
-- Mod: Animal Worker
--
-- @Author: Tunnelrat
-- @Date: 06.25.2019
-- @Version: 2.2 Fixed typo, optimized; added inputBinding and menu to toggle worker settings. Now in 4 languages.
-- Copyright (C) Tunnelrat, All Rights Reserved.

--[[
	TO DO: Add gui settings
	 - fix help display toggle to show current worker status (?)
]]--

animalWorker = {};

local modDesc = loadXMLFile("modDesc", g_currentModDirectory .. "modDesc.xml")

animalWorkerActive = true -- if true, the Ranch Hand will work everyday, set to false to disable. Default: true
animalWorkerFee = 0.05 -- convenience fee percentage for doing all this work. Default: 0.05
costPerAnimal = 10.00 -- base cost to tend to each animal. Default: 10.00
fillToCapacity = true -- false if you want to give enough food,water,straw for 6 days. Default: true
cleanPens = true -- if true, will also clean the animal pens once a day. Default: true
helpDisplay = "awActive"
--activeInput = false

animalWorker.version = getXMLString(modDesc, "modDesc.version")
animalWorker.modDirectory = g_currentModDirectory

animalWorker.fees = {0.05, 0.10, 0.15, 0.20, 0.25, 0.30, 0.35, 0.40, 0.45, 0.50}
animalWorker.costs = {1.00, 2.50, 5.00, 10.00, 15.00, 20.00, 25.00, 50.00, 100.00, 250.00}

addModEventListener(animalWorker)

function animalWorker:loadMap()

-- Player action event registration
Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, animalWorker.registerActionEventsPlayer)
Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, animalWorker.removeActionEventsPlayer)	

print("#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*")
print("          Animal Worker v " .. self.version .. " by Tunnelrat loaded")
print("#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*#*")

-- Settings
animalWorker.settingsDir = getUserProfileAppPath().. "modsSettings/animalWorker/"

-- Gui Settings
animalWorker.gui = {};
animalWorker.gui["awSettingGui"] = awGui:new();
g_gui:loadGui(animalWorker.modDirectory .. "gui/awGui.xml", "awGui", animalWorker.gui.awSettingGui);

-- Actions and Events
animalWorker:LoadSettings()
animalWorker:SaveSettings()
animalWorker.eventName = {}

end

function animalWorker:SaveSettings()

	createFolder(getUserProfileAppPath() .. "modsSettings/")
	createFolder(animalWorker.settingsDir)
	local file = animalWorker.settingsDir .. "animalWorker.xml"
	local pos = 0
	local rootPath = "animalWorker.settings"
	local groupNameTag = string.format("%s.setting(%d)", rootPath, pos)

	if not fileExists(file) then
		-- create
		local xmlContent = createXMLFile("animalWorker_XML", file, "animalWorker")
		
		setXMLBool(xmlContent, groupNameTag .. "#animalWorkerActive", animalWorkerActive)
		setXMLFloat(xmlContent, groupNameTag .. "#animalWorkerFee", animalWorkerFee)
		setXMLFloat(xmlContent, groupNameTag .. "#costPerAnimal", costPerAnimal)
		setXMLBool(xmlContent, groupNameTag .. "#fillToCapacity", fillToCapacity)
		setXMLBool(xmlContent, groupNameTag .. "#cleanPens", cleanPens)
		saveXMLFile(xmlContent)
		delete(xmlContent)
	else
		-- update
		local xmlContent = createXMLFile("animalWorker_XML", file, "animalWorker")
		
		setXMLBool(xmlContent, groupNameTag .. "#animalWorkerActive", animalWorkerActive)
		setXMLFloat(xmlContent, groupNameTag .. "#animalWorkerFee", animalWorkerFee)
		setXMLFloat(xmlContent, groupNameTag .. "#costPerAnimal", costPerAnimal)
		setXMLBool(xmlContent, groupNameTag .. "#fillToCapacity", fillToCapacity)
		setXMLBool(xmlContent, groupNameTag .. "#cleanPens", cleanPens)
		saveXMLFile(xmlContent)
		delete(xmlContent)
	end

end

function animalWorker:LoadSettings()
	
	local file = animalWorker.settingsDir .. "animalWorker.xml"
	
	if fileExists(file) then
		local xmlContent = loadXMLFile("animalWorker_XML", file, "animalWorker")
		
		local pos = 0
		local rootPath = "animalWorker.settings"
		local groupNameTag = string.format("%s.setting(%d)", rootPath, pos)
		
		animalWorkerActive 	= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#animalWorkerActive"), true)
		animalWorkerFee		= Utils.getNoNil(getXMLFloat(xmlContent, groupNameTag .. "#animalWorkerFee"), 0.05)
		costPerAnimal		= Utils.getNoNil(getXMLFloat(xmlContent, groupNameTag .. "#costPerAnimal"), 10.00)
		fillToCapacity		= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#fillToCapacity"), true)
		cleanPens			= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#cleanPens"), true)
	else
		print("Animal Worker XML settings not found.")
	end
	
end

function animalWorker:delete()

end

function animalWorker:registerActionEventsPlayer()
	-- Animal Worker Toggle (true/false)
	local r, eventName = InputBinding.registerActionEvent(g_inputBinding, 'animalWorker_Active', self, animalWorker.actionAnimalWorkerActive, false, true, false, true)
	if r then
		table.insert(animalWorker.eventName, eventName)
		g_inputBinding.events[eventName].displayIsVisible = true
		g_inputBinding:setActionEventText(eventName, g_i18n:getText("input_animalWorker_Active"))
    end
	-- Animal Worker Settings
	local r, eventName = InputBinding.registerActionEvent(g_inputBinding, 'animalWorker_Setting', self, animalWorker.actionAnimalWorker_Setting, false, true, false, true)
	if r then
		table.insert(animalWorker.eventName, eventName)
		g_inputBinding.events[eventName].displayIsVisible = true
		g_inputBinding:setActionEventText(eventName, g_i18n:getText("input_animalWorker_Setting"))
	end
	--print("Animal Worker: Registering input events...")
end

function animalWorker:removeActionEventsPlayer()
	animalWorker.eventName = {}
end

function animalWorker:actionAnimalWorkerActive(actionName)
	if actionName == "animalWorker_Active" then -- key pressed
		-- local eventId = animalWorker.eventName[InputAction.actionName]
		if animalWorkerActive then
			animalWorkerActive = false
			g_inputBinding:setActionEventText(actionName, g_i18n:getText("awNotActive"))
		elseif not animalWorkerActive then
			animalWorkerActive = true
			g_inputBinding:setActionEventText(actionName, g_i18n:getText("awActive"))
		end
	end
	animalWorker:SaveSettings()
	print("Animal Worker: Saving Settings")
end

-- Show settings menu
function animalWorker:actionAnimalWorker_Setting(actionName, keyStatus, arg3, arg4, arg5)
	if animalWorker.gui.awSettingGui.isOpen then
		animalWorker.gui.awSettingGui:onClickBack()
	elseif g_gui.currentGui == nil then
		g_gui:showGui("awGui")
	end
end

function animalWorker:keyEvent(unicode, sym, modifier, isDown)

end

function animalWorker:mouseEvent(posX, posY, isDown, isUp, button)
end

function animalWorker:deleteMap()
end

function animalWorker:updateGui()
	-- Worker Status
	animalWorker.gui.awSettingGui:setActiveStatus(animalWorkerActive)
	
	-- Worker Fee
	index = 0
	for i = 1, #animalWorker.fees, 1 do
		if (animalWorker.fees[i] == animalWorkerFee) then
			index = i;
			break
		end
	end
	if (index ~= 0) then
		animalWorker.gui.awSettingGui:setWorkerFee(index);
	end
	
	-- Worker Cost
	index = 0
	for i = 1, #animalWorker.costs, 1 do
		if (animalWorker.costs[i] == costPerAnimal) then
			index = i;
			break
		end
	end
	if (index ~= 0) then
		animalWorker.gui.awSettingGui:setWorkerCost(index);
	end
	
	-- Worker Capacity
	animalWorker.gui.awSettingGui:setCapacity(fillToCapacity)
	
	-- Worker Pens
	animalWorker.gui.awSettingGui:setCleanPens(cleanPens)
end

function animalWorker:settingsFromGui(sActive, sFee, sCost, sCap, sPen)
	-- Update settings
	animalWorkerActive = sActive
	animalWorkerFee = animalWorker.fees[sFee]
	costPerAnimal = animalWorker.costs[sCost]
	fillToCapacity = sCap
	cleanPens = sPen
	-- Save to xml
	animalWorker:SaveSettings()
	animalWorker:LoadSettings()
	--print("Animal Worker: Debug settingsFromGui() in order " .. tostring(sActive) .. " " .. animalWorker.fees[sFee] .. " " .. animalWorker.costs[sCost] .. " " .. tostring(sCap) .. " " .. tostring(sPen) .. ".")
	--print("Animal Worker: Debug new values from XML " .. tostring(animalWorkerActive) .. " " .. animalWorkerFee .. " " .. costPerAnimal .. " " .. tostring(fillToCapacity) .. " " .. tostring(cleanPens) .. " .")
end

function animalWorker:update(dt)
	
	local hourNow = g_currentMission.environment.currentHour
	local minuteNow = g_currentMission.environment.currentMinute
	
	farmData = {
		ownerFarmId = 0,
		farmId = 0,
	}
	
	local playerOwnsFarm = false
	
	function checkIsOwnerOfFarm(husbandry)
		if g_currentMission:getIsClient() then
			farmData.ownerFarmId = g_currentMission.player.ownerFarmId
			farmData.farmId = g_currentMission.player.farmId
			
			if husbandry.modulesByName.animals.owner.ownerFarmId == farmData.farmId then
				playerOwnsFarm = true
				return true
			else
				playerOwnsFarm = false
				return false
			end
		end
	end
	
	function animalCount(husbandry)
		if husbandry ~= nil then
			local animals = husbandry.modulesByName.animals.animals
			local numAnimals = 0
			for k,v in pairs(animals) do
				numAnimals = tonumber(k)
			end
			return numAnimals
		else
			print("animalWorker:bad parameters for animalCount()")
			return 0
		end
	end
	--[[FillTypes concatenated
	local wheat = 2	local barley = 3	local canola = 5	local forage = 19	local silage = 24	local hay = 30
	local grass = 28	local sunflower = 6	local soybean = 7 -- g_currentMission.fillTypeManager:getFillTypeIndexByName("fillType_soybean")
	local maize = 8	local oat = 4	local potato = 9	local sugerbeet = 10]]--
	local chickens, cows, horse, pigs, sheep, all = 0, 0, 0, 0, 0, 0
	local eatDayChicken = 5
	local eatDayCow = 350
	local eatDayHorse = 400
	local eatDayPig = 90
	local eatDaySheep = 50
	
	-- Fill water troughs, food and straw at 7am if worker is active
	if hourNow == 7 and minuteNow == 00 and animalWorkerActive and not self.doOnce then
		
		-- Run one time
		self.doOnce = true
		-- FOR LOOP
		for _,husbandry in pairs(g_currentMission.husbandries) do
		
			local animalType = husbandry.modulesByName.animals.animalType
		
			if checkIsOwnerOfFarm(husbandry) == true then
				
				-- Clean Pens
				if cleanPens == true then
					if husbandry.modulesByName.foodSpillage ~= nil then
						husbandry.modulesByName.foodSpillage.cleanlinessFactor = 1.0
						husbandry:setModuleParameters("foodSpillage", 0, 0)
					end
				end
					
				-- Water
				if husbandry.modulesByName.water ~= nil then
					local amountToAdd = husbandry.modulesByName.water:getFreeCapacity(FillType.WATER)
					local fillLevel, capacity = 0, 0
    
					fillLevel = husbandry.modulesByName.water:getFillLevel(FillType.WATER)
					capacity = husbandry.modulesByName.water.fillCapacity
		
					if amountToAdd > 0 then
						if fillToCapacity == true then
							husbandry.modulesByName.water:setFillLevel(FillType.WATER, capacity)					
						else
							local fillTo = 0
							if animalType ~= nil and animalType == "CHICKEN" then -- base game gives water consumption amount, use for mods/updates
								chickens = animalCount(husbandry)
								fillTo = tonumber(math.ceil(chickens * 7 * 6)) -- 7 L per chicken (6 days)
							elseif animalType ~= nil and animalType == "COW" then
								cows = animalCount(husbandry)
								fillTo = tonumber(math.ceil(cows * 80 * 6)) -- 80 L per cow (6 days)
							elseif animalType ~= nil and animalType == "HORSE" then
								horse = animalCount(husbandry)
								fillTo = tonumber(math.ceil(horse * 50 * 6)) -- 50 L per horse (6 days)
							elseif animalType ~= nil and animalType == "PIG" then
								pigs = animalCount(husbandry)
								fillTo = tonumber(math.ceil(pigs * 20 * 6)) -- 20 L per pig (6 days)
							elseif animalType ~= nil and animalType == "SHEEP" then
								sheep = animalCount(husbandry)
								fillTo = tonumber(math.ceil(sheep * 20 * 6)) -- 20 L per sheep (6 days)
							end
							husbandry.modulesByName.water:setFillLevel(FillType.WATER, fillTo)
						end
					end
				end
				
				-- Straw
				if husbandry.modulesByName.straw ~= nil then
					local amountToAdd = husbandry.modulesByName.straw:getFreeCapacity(FillType.STRAW)
					local fillLevel, capacity = 0, 0
    
					fillLevel = husbandry.modulesByName.straw:getFillLevel(FillType.STRAW)
					capacity = husbandry.modulesByName.straw.fillCapacity
		
					if amountToAdd > 0 then
						if fillToCapacity == true then
							husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, capacity)					
						else
							local fillTo = 0
							if animalType ~= nil and animalType == "CHICKEN" then
								-- Do nothing, 0 straw
							elseif animalType ~= nil and animalType == "COW" then
								cows = animalCount(husbandry)
								fillTo = tonumber(math.ceil(cows * 95 * 6)) -- 95 L per cow (6 days)
							elseif animalType ~= nil and animalType == "HORSE" then
								horse = animalCount(husbandry)
								fillTo = tonumber(math.ceil(horse * 80 * 6)) -- 80 L per horse (6 days)
							elseif animalType ~= nil and animalType == "PIG" then
								pigs = animalCount(husbandry)
								fillTo = tonumber(math.ceil(pigs * 30 * 6)) -- 30 L per pig (6 days)
							elseif animalType ~= nil and animalType == "SHEEP" then
								-- Do nothing, 0 straw
							end
							husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, fillTo)
						end
					end
				end
				
				-- Chicken
				if animalType ~= nil and animalType == "CHICKEN" then
					chickens = animalCount(husbandry)
					
					-- Food					
					if chickens > 0 then
						local foodToAdd = 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(chickens * 5 * 6)) -- numAnimals * litersFoodPerAnimal * dayToInterval
						else
							foodToAdd = tonumber(fCap)
						end
						-- print("animalWorker: Fed " .. foodToAdd .. " wheat and barley to " .. chickens .. " chickens.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.WHEAT, foodToAdd)
							husbandry.modulesByName.food:setFillLevel(FillType.BARLEY, foodToAdd)
						end
					end
				end	-- End chicken
				
				-- Cow
				if animalType ~= nil and animalType == "COW" then
					cows = animalCount(husbandry)
					
					-- Food
					if cows > 0 then
						local foodToAdd, f75, f25 = 0, 0, 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(cows * 500 * 6))
							f75 = tonumber(math.floor(cows * math.ceil(350 * 0.75) * 6)) -- 75% eat weight
							f25 = tonumber(math.floor(cows * math.ceil(350 * 0.25) * 6)) -- 25%
						else
							f75 = tonumber(math.ceil(fCap * 0.75))
							f25 = tonumber(math.ceil(fCap * 0.25))
							foodToAdd = tonumber(fCap)
						end
						-- print("animalWorker: Fed " .. foodToAdd .. " TMR, " .. f75 .. " silage and hay, " .. f25 .. " grass to " .. cows .. " cows.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.FORAGE, foodToAdd)
							husbandry.modulesByName.food:setFillLevel(FillType.SILAGE, foodToAdd) --f75)
							husbandry.modulesByName.food:setFillLevel(FillType.DRYGRASS_WINDROW, foodToAdd) --f75)
							husbandry.modulesByName.food:setFillLevel(FillType.GRASS_WINDROW, foodToAdd) --f25)
						end
					end
					
				end -- End cow
				
				-- Horse
				if animalType ~= nil and animalType == "HORSE" then
					horse = animalCount(husbandry)
					
					-- Food
					if horse > 0 then
						local foodToAdd, f60, f40 = 0, 0, 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(horse * 400 * 6))
							f60 = tonumber(math.floor(horse * math.ceil(400 * 0.6) * 6)) -- 60% eat weight
							f40 = tonumber(math.floor(horse * math.ceil(400 * 0.4) * 6)) -- 40% eat weight
						else
							f60 = tonumber(math.ceil(fCap * 0.6))
							f40 = tonumber(math.ceil(fCap * 0.4))
							foodToAdd = tonumber(fCap)
						end
						-- print("animalWorker: Fed " .. foodToAdd .. " oats and hay to " .. horse .. " horses.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.OAT, foodToAdd)
							husbandry.modulesByName.food:setFillLevel(FillType.DRYGRASS_WINDROW, foodToAdd)
						end
					end
				end -- End horse
				
				-- Pig
				if animalType ~= nil and animalType == "PIG" then
					pigs = animalCount(husbandry)
					
					-- Food
					if pigs > 0 then
						local foodToAdd, f50, f25, f20, f5 = 0, 0, 0, 0, 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(pigs * 90 * 6)) -- 100%
							f50 = tonumber(math.floor(pigs * math.ceil(90 * 0.5) * 6)) -- 50%
							f25 = tonumber(math.floor(pigs * math.ceil(90 * 0.25) * 6)) -- 25%
							f20 = tonumber(math.floor(pigs * math.ceil(90 * 0.2) * 6)) -- 20%
							f5 = tonumber(math.floor(pigs * math.ceil(90 * 0.05) * 6)) -- 5%
						else
							f50 = tonumber(math.ceil(fCap * 0.5))
							f25 = tonumber(math.ceil(fCap * 0.25))
							f20 = tonumber(math.ceil(fCap * 0.2))
							f5 = tonumber(math.ceil(fCap * 0.05))
							foodToAdd = tonumber(fCap)
						end
						-- print("animalWorker: Fed (" .. foodToAdd .. " total) " .. f50 .. " corn, " .. f25 .. " grains, " .. f20 .. " proteins, " .. f5 .. " roots to " .. pigs .. " pigs.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.MAIZE, foodToAdd) --f50)
							husbandry.modulesByName.food:setFillLevel(FillType.WHEAT, foodToAdd) --f25)
							husbandry.modulesByName.food:setFillLevel(FillType.BARLEY, foodToAdd) --f25)
							husbandry.modulesByName.food:setFillLevel(FillType.SOYBEAN, foodToAdd) --f20)
							husbandry.modulesByName.food:setFillLevel(FillType.CANOLA, foodToAdd) --f20)
							husbandry.modulesByName.food:setFillLevel(FillType.SUNFLOWER, foodToAdd) --f20)
							husbandry.modulesByName.food:setFillLevel(FillType.POTATO, foodToAdd) --f5)
							husbandry.modulesByName.food:setFillLevel(FillType.SUGARBEET, foodToAdd) --f5)
						end
					end
				end -- End pig
				
				-- Sheep
				if animalType ~= nil and animalType == "SHEEP" then
					sheep = animalCount(husbandry)
					-- Food
					if sheep > 0 then
						local foodToAdd = 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(sheep * 50 * 6)) -- 100%
						else
							foodToAdd = tonumber(fCap)
						end
						-- print("animalWorker: Fed " .. foodToAdd .. " grass to " .. sheep .. " sheep.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.GRASS_WINDROW, foodToAdd)
						end
					end
				end -- End sheep
			end -- owner
		
		end -- end for loop
		
		all = tonumber(chickens + cows + pigs + horse + sheep)
		
		-- Pay the worker...
		local difficultyMultiplier = g_currentMission.missionInfo.buyPriceMultiplier
		-- local waterCost = g_currentMission.economyManager:getCostPerLiter(FillType.WATER, false)
		local moneyUnit = g_gameSettings.moneyUnit
		local showCurrency = "DOLLARS"
		if moneyUnit == "1" then
			showCurrency = "EUROS"
		elseif moneyUnit == "2" then
			showCurrency = "DOLLARS"
		elseif moneyUnit == "3" then
			showCurrency = "POUNDS"
		end
		local price = 0
		local farmCost = 0
		if all ~= 0 then
			price = tonumber(math.ceil(costPerAnimal * all * difficultyMultiplier)) -- 10 * numAnimals * {1.0, 1.8, 3.0}
			price = price + (math.floor(price * animalWorkerFee)) -- sum with fee
			farmCost = tonumber(math.floor(price))
			-- print("animalWorker:Cost " .. farmCost .. " total.")
			if farmCost ~= 0 then
				g_currentMission:farmStats(farmData.farmId):updateStats("expenses", farmCost)
				g_currentMission:addMoney(-farmCost, farmData.farmId, MoneyType.ANIMAL_UPKEEP, true)
				print("Animal Worker: Ranch hand fed, watered and bedded all " .. all .. " animals at 7am for " .. farmCost .. " " .. showCurrency .. ".")
				doOnce = false
			end
		end	-- End costs
		
	end
end

function animalWorker:draw()
end